package storage

import (
	"bitbucket.org/orus-io/go-xbus/api"
)

// JobID A job unique id.
type JobID struct {
	ProcessID api.UUID
	Node      ProcessNode
}

// JobState contains all the job state
type JobState struct {
	Detached bool
	Status   api.ActorProcessingState_Status
	// XXX add a timestamp, a sort id
}

// Job is a director job definition and state
type Job struct {
	JobID
	JobState
}

// JobStorage persists the jobs
type JobStorage interface {
	NewJob(JobID, JobState)
	SetState(JobID, JobState)
	GetState(JobID) JobState

	ListPendingJobByActor(actor api.UUID, after *JobID, limit int) []Job
	ListJobByProcess(api.UUID) []Job

	Purge(api.UUID) error
}
